@@ -4,7 +4,7 @@
 /*                                                              */
 /* BGP Datainformation Routine                                  */
 /****************************************************************/
-/* bgpdata.c,v 1.6 2001/09/26 07:21:22 kuniaki Exp */
+/* bgpdata.c,v 1.7 2002/04/09 09:43:03 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -32,10 +32,7 @@
  * DISCLAIMED.
  */
 
-#include "bgp.h"
-#include <time.h>
 #include "bgpdata.h"
-#include "bgplog.h"
 
 BGP_INFO	bgpinfo[MAXPEERS];
 
