@@ -4,7 +4,7 @@
 /*                                                              */
 /* Logging  Routine                                             */
 /****************************************************************/
-/* bgplog.c,v 1.10 2001/09/16 12:01:33 kuniaki Exp */
+/* bgplog.c,v 1.11 2002/04/09 09:43:03 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2001 Internet Initiative Japan Inc.
@@ -32,12 +32,7 @@
  * DISCLAIMED.
  */
 
-#include "bgp.h"
-#include <syslog.h>
-#include <varargs.h>
 #include "bgplog.h"
-#include "timer.h"
-#include "route.h"
 
 /* System Log File Poingter */
 FILE	*sfd;
@@ -273,3 +268,59 @@ int		pn;	/* Peer number */
 	}
 	return 0;
 }
+
+int write_v6prefixlog(int typ, void *prefix, int pn)
+/*
+int		typ;	/* 0:Update 1:Withdraw 
+void		*prefix;
+int		pn;	/* Peer number 
+*/
+{
+	V6ROUTE_BUF	*withdrawn;
+	ROUTE_INFO_V6	*update;
+	struct in_addr	inaddr;
+	char		v6addr[IPV6_ADDR_LEN];
+	char		lbuf[50], lbuf2[50];
+	char		stmp[4096];
+	V6ROUTE_LIST	*v6list;
+
+	if (peer[pn].pfd == NULL) return 1;
+	pr_time(peer[pn].pfd);
+	if (typ == 0) {
+		/* Update */
+		fprintf(peer[pn].pfd, "UPDATE  : ");
+		update = (ROUTE_INFO_V6 *)prefix;
+		bintostrv6(update->prefix, lbuf2);
+		sprintf(lbuf, "%s/%d", lbuf2, update->length);
+		if (update->nexthop_afi == AF_IPV4) {
+			inaddr.s_addr = *(net_ulong *)update->next_hop;
+			strcpy(lbuf2, inet_ntoa(inaddr));
+		} else if (update->nexthop_afi == AF_IPV6) {
+			bintostrv6(update->next_hop, lbuf2);
+		} else {
+			strcpy(lbuf2, "::");
+		}
+		if (update->pathset_ptr == NULL) {
+			stmp[0] = '\0';
+		} else {
+			strcpy(stmp, update->pathset_ptr);
+		}
+		fprintf(peer[pn].pfd, "%-18s %-15s %s\n",
+			lbuf,
+			lbuf2,
+			/* update->path_set); */
+			stmp);
+	} else {
+		/* WITHDRAW */
+		fprintf(peer[pn].pfd, "WITHDRAW: ");
+		withdrawn = (V6ROUTE_BUF *)prefix;
+		v6list = withdrawn->preflist;
+		while(v6list != NULL) {
+			bintostrv6(v6list->v6prefix, lbuf2);
+			fprintf(peer[pn].pfd, "%s/%d\n", lbuf2,
+							 v6list->length);
+			v6list = v6list->next;
+		}
+	}
+	return 0;
+}
