@@ -4,7 +4,7 @@
 /*                                                              */
 /* Main Routine                                                 */
 /****************************************************************/
-/* bgpview.c,v 1.46 2002/04/01 10:01:34 kuniaki Exp */
+/* bgpview.c,v 1.48 2002/04/09 09:43:04 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2002 Internet Initiative Japan Inc.
@@ -33,17 +33,6 @@
  */
 
 #include "bgp.h"
-#include <signal.h>
-#include <errno.h>
-#include <fcntl.h>
-#include "bgpdata.h"
-#include "bgplog.h"
-#include "timer.h"
-#include "dummyroute.h"
-
-#ifdef RADIX
-#include "radix.h"
-#endif
 
 #ifdef sun
 #define PID	"/var/adm/bgpview.pid"
@@ -530,6 +519,15 @@ int sendopen(int pn)
 		*(buf+bt+35) = 0;
 		*(buf+bt+36) = SAFI_NLRI_UNI;
 		bt += 8;
+	} if (peer[pn].nlri == IPv4_UNICAST) {
+		*(buf+bt+29) = BGPOPT_CAPABILITY;
+		*(buf+bt+30) = 6;
+		*(buf+bt+31) = CAP_MPBGP;
+		*(buf+bt+32) = 4;
+		store_int16(buf+33, AF_IPV4);
+		*(buf+bt+35) = 0;
+		*(buf+bt+36) = SAFI_NLRI_UNI;
+		bt += 8;
 	}
 
 	*(buf+28) = bt;
