@@ -3,7 +3,7 @@
 /*                                                              */
 /* BGPView Shell Commands                                       */
 /****************************************************************/
-/* bvcommand.c,v 1.53 2001/10/01 11:39:38 kuniaki Exp */
+/* bvcommand.c,v 1.54 2002/04/09 09:43:04 kuniaki Exp */
 
 /*
  * Copyright (c) 2000 - 2001 Internet Initiative Japan Inc.
@@ -31,31 +31,7 @@
  * DISCLAIMED.
  */
 
-#include <sys/param.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <arpa/telnet.h>
-#include <sys/time.h>
-#include <netdb.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <errno.h>
 #include "bvcommand.h"
-#include "bgp.h"
-#include "bvshell.h"
-#include "bgplog.h"
-#include "bgpdata.h"
-#include "dummyroute.h"
-#include "mailannounce.h"
-#include "timer.h"
-
-#ifdef RADIX
-#include "radix.h"
-#endif
 
 /****** Functions **********/
 int bvc_shutdown(BVS_CMD_PARAM *prm)
