@@ -4,7 +4,7 @@
 /*                                                              */
 /* BGPView Shell Commands Header File                           */
 /****************************************************************/
-/* bvcommand.h,v 1.29 2001/09/26 07:21:22 kuniaki Exp */
+/* bvcommand.h,v 1.30 2002/04/09 09:43:04 kuniaki Exp */
 
 /*
  * Copyright (c) 2000-2001 Internet Initiative Japan Inc.
@@ -40,13 +40,27 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <arpa/telnet.h>
 #include <sys/time.h>
 #include <netdb.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include "bgp.h"
+#include <errno.h>
+#include "bvtypes.h"
+/* #include "bgp.h" */
+#include "bvshell.h"
+#include "bgplog.h"
+#include "bgpdata.h"
+#include "dummyroute.h"
+#include "mailannounce.h"
+#include "timer.h"
+
+#ifdef RADIX
+#include "radix.h"
+#endif
+
 
 /* Defines */
 
@@ -59,50 +73,6 @@
 #define RTV_ACTIVE	1
 #define RTV_INACTIVE	2
 
-typedef union {
-	struct {
-		int	clno;
-	} noparam;
-	struct {
-		int	clno;
-		u_long	addr;
-	} adrs;
-	struct {
-		int	clno;
-		u_long	neighbor;
-		u_long	prefix;
-		int	prefixlen;
-		int	timeout;
-	} routertt;
-	struct {
-		int	clno;
-		int	num;
-	} one_num;
-	struct {
-		int	clno;
-		u_long	prefix;
-		int	prefixlen;
-	} one_route;
-	struct {
-		int	clno;
-		char	str[64];
-	} one_str64;
-	struct {
-		int	clno;
-		char	str[64];
-		u_long	addr;
-	} one_str64_adrs;
-	struct {
-		int	clno;
-		char	longstr[128];
-	} longstr128;
-	struct {
-		int	clno;
-		u_long	addr;
-		boolean	swc;
-	} adrs_boolean;
-} BVS_CMD_PARAM;
-
 typedef struct nexthop_list {
 	struct nexthop_list	*next;
 	u_long			next_hop;
