@@ -4,7 +4,7 @@
 /*                                                              */
 /* Start Configuration Routine                                  */
 /****************************************************************/
-/* config.c,v 1.29 2002/01/31 05:05:46 kuniaki Exp */
+/* config.c,v 1.30 2002/03/29 03:24:02 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2002 Internet Initiative Japan Inc.
@@ -41,6 +41,30 @@
 #include "dummyroute.h"
 #include "mailannounce.h"
 #include "timer.h"
+#include "mpnlri.h"
+
+int check_nlritype(char *nlristr)
+{
+	int	cnt;
+	int	ret;
+	char	nlstr[LOGSTRSIZE];
+
+	strcpy(nlstr, nlristr);
+	cnt = 0;
+	while(nlstr[cnt] != '\0') {
+		nlstr[cnt] = toupper(nlstr[cnt]);
+		cnt++;
+	}
+	
+	if (strcmp(nlstr, "IPV4_UNICAST") == 0) {
+		ret = IPv4_UNICAST;
+	} else if (strcmp(nlstr, "IPV6_UNICAST") == 0) {
+		ret = IPv6_UNICAST;
+	} else {
+		ret = 0;
+	}
+	return(ret);
+}
 
 int read_conf(fname)
 char *fname;
@@ -193,6 +217,7 @@ char *fname;
 			peer[pn].datalen   = 0;
 			peer[pn].soc       = -1;
 			peer[pn].flag	   = 0;
+			peer[pn].nlri      = 0;
 			peer[pn].status    = IDLE;
 			peer[pn].event     = BGP_START;
 			peer[pn].holdtimer = HOLD_TIMER;
@@ -230,6 +255,8 @@ char *fname;
 		} else if (strcmp("KEEPALIVE", cmd) == 0) {
 			if (val == NULL) continue;
 			sscanf(val, "%d", &peer[pn].keepalive);
+		} else if (strcmp("NLRI", cmd) == 0) {
+			peer[pn].nlri = check_nlritype(val);
 		} else if (strcmp("LOGNAME", cmd) == 0) {
 			if (val == NULL) continue;
 			strcpy(peer[pn].logname, val);
