@@ -4,7 +4,7 @@
 /*                                                              */
 /* FSM Routine                                                  */
 /****************************************************************/
-/* fsm.c,v 1.12 2002/03/28 09:54:36 kuniaki Exp */
+/* fsm.c,v 1.14 2002/04/09 09:43:05 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -32,10 +32,11 @@
  * DISCLAIMED.
  */
 
-#include "bgp.h"
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/time.h>
+#include "bvtypes.h"
+#include "bgp.h"
 #include "bgplog.h"
 #include "timer.h"
 
@@ -113,14 +114,19 @@ char capability_check(char *param, char length, int pn)
 
 			switch(ntohs(*(u_short *)(ptr+2))) {
 
-#ifdef HAVE_IPv6
+			case AF_IPV4:
+#ifdef DEBUG
+				printf("DEBUG: CAPACHK: Address Family IPv4\n");
+#endif
+				opterr = 0;
+				break;			
+
 			case AF_IPV6:
 #ifdef DEBUG
 				printf("DEBUG: CAPACHK: Address Family IPv6\n");
 #endif
 				opterr = 0;
 				break;
-#endif
 			default:
 				pr_log(VLOG_GLOB, pn, "CAPACHK: AFI_CHK: Unsupported Capability\n", ON);
 				opterr = UNSUPPORTED_CAPABILITY;
