@@ -4,7 +4,7 @@
 /*                                                              */
 /* Routeing Information Update Routine                          */
 /****************************************************************/
-/* update.c,v 1.23 2002/04/01 10:01:35 kuniaki Exp */
+/* update.c,v 1.25 2002/04/09 09:57:01 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -35,10 +35,10 @@
 #include "bgp.h"
 #include "bgpdata.h"
 #include "bgplog.h"
-#include "route.h"
 #include "routertt.h"
 #include "timer.h"
 #include "mpnlri.h"
+#include "route_ipv6.h"
 
 #ifdef RADIX
 #include "radix.h"
@@ -158,6 +158,74 @@ void pr_route(ROUTE_INFO *routep, int pn)
 	pr_log(VLOG_UPDE, pn, lbuf);
 }
 
+void pr_v6route(ROUTE_INFO_V6 *routep, int pn)
+{
+	struct in_addr	inaddr;
+	char		v6addr[IPV6_ADDR_LEN];
+	char		lbuf[1024], lbuf2[1024], lbuf3[1024];
+	struct timeval	now;
+	long		timediff;
+	int		cnt;
+
+	gettimeofday(&now, (struct timezone *)0);
+	timediff = now.tv_sec - routep->lastupdate.tv_sec;
+
+	bintostrv6(routep->prefix, lbuf3);
+	sprintf(lbuf2, "%s/%d ", lbuf3, routep->length);
+	sprintf(lbuf, "   %-17s", lbuf2);
+	
+	if (ATRFL_ISSET(routep->attribute_flag, SET_NEXTHOP)) {
+		if (routep->nexthop_afi == AF_IPV4) {
+			if ((inaddr.s_addr = *(net_ulong *)routep->next_hop)==0){
+				bintostrv6(routep->next_hop, lbuf3);
+				sprintf(lbuf2, "%-17s", lbuf3);
+			} else {
+				sprintf(lbuf2, "%-17s", inet_ntoa(inaddr));
+			}
+		} else if (routep->nexthop_afi == AF_IPV6) {
+			bintostrv6(routep->next_hop, lbuf3);
+			sprintf(lbuf2, "%-17s", lbuf3);
+		} else {
+			strcpy(lbuf2, "                 ");
+		}
+	} else {
+		strcpy(lbuf2, "                 ");
+	}
+	strcat(lbuf, lbuf2);
+	if (ATRFL_ISSET(routep->attribute_flag, SET_METRIC)) {
+		sprintf(lbuf2, "%7d", routep->metric);
+	} else {
+		strcpy(lbuf2, "       ");
+	}
+	strcat(lbuf, lbuf2);
+	if (ATRFL_ISSET(routep->attribute_flag, SET_LOCALPREF)) {
+		sprintf(lbuf2, "%7d", routep->local_perf);
+	} else {
+		strcpy(lbuf2, "       ");
+	}
+	strcat(lbuf, lbuf2);
+	sprintf(lbuf2, "%7d ", timediff);
+	strcat(lbuf, lbuf2);
+	/* strcat(lbuf, routep->path_set); */
+	if (routep->pathset_ptr != NULL) {
+		strcat(lbuf, routep->pathset_ptr);
+	}
+	if (ATRFL_ISSET(routep->attribute_flag, SET_ORIGIN)) {
+		switch(routep->origin_type) {
+		case ORIGIN_IGP:
+			strcat(lbuf, "i\n");
+			break;
+		case ORIGIN_EGP:
+			strcat(lbuf, "e\n");
+			break;
+		case ORIGIN_INCOMPLETE:
+			strcat(lbuf, "?\n");
+			break;
+		}
+	}
+	pr_log(VLOG_UPDE, pn, lbuf);
+}
+
 void pr_update(int pn)
 {
 	char 		*ptr, *ptr2;
@@ -173,6 +241,9 @@ void pr_update(int pn)
 	char		pathbuf[4096];
 	int		pathbuflen;
 	int		ipv6updcnt, ipv6wdrcnt;
+	V6ROUTE_BUF	v6update, v6withdraw;
+	ROUTE_INFO_V6	v6getroute, v6prroute;
+	V6ROUTE_LIST	*listbuf;
 
 	static char	*path_attr[] = {
 			/* 0 */ "UNKNOWN",
@@ -197,10 +268,16 @@ void pr_update(int pn)
 
 	ptr = peer[pn].data;
 
+	bzero(&v6update, sizeof(V6ROUTE_BUF));
+	bzero(&v6withdraw, sizeof(V6ROUTE_BUF));
+	ipv6updcnt = 0;
+	ipv6wdrcnt = 0;
+
 	pr_log(VLOG_UPDE, pn, "====== UPDATE Information =====\n", ON);
 
 	message_dump(pn);
 	bzero(&getroute, sizeof(ROUTE_INFO));
+	bzero(&v6getroute, sizeof(ROUTE_INFO_V6));
 	
 	length = (int)buf2ushort(ptr);
 	le1 = length;
@@ -287,24 +364,30 @@ void pr_update(int pn)
 			if (*ptr & 0x80) {
 				strcpy(logstr, "    Attribute : OPTIONAL ");
 				getroute.attr_type = OPTIONAL;
+				v6getroute.attr_type = OPTIONAL;
 			} else {
 				getroute.attr_type = WELL_KNOWN;
+				v6getroute.attr_type = WELL_KNOWN;
 				strcpy(logstr, "    Attribute : WELL-KNOWN ");
 			}
 
 			if (*ptr & 0x40) {
 				getroute.transitive = TRANSITIVE;
+				v6getroute.transitive = TRANSITIVE;
 				strcat(logstr, "TRANSITIVE ");
 			} else {
 				getroute.transitive = NON_TRANSITIVE;
+				v6getroute.transitive = NON_TRANSITIVE;
 				strcat(logstr, "NON-TRANSITIVE ");
 			}
 
 			if (*ptr & 0x20) {
 				getroute.partial = PARTIAL;
+				v6getroute.partial = PARTIAL;
 				strcat(logstr, "PARTIAL ");
 			} else {
 				getroute.partial = COMPLETE;
+				v6getroute.partial = COMPLETE;
 				strcat(logstr, "COMPLETE ");
 			}
 
@@ -348,8 +431,10 @@ void pr_update(int pn)
 				break;
 			case ATTR_ORIGIN:
 				ATRFL_SET(SET_ORIGIN, getroute.attribute_flag);
+				ATRFL_SET(SET_ORIGIN, v6getroute.attribute_flag);
 				strcpy(logstr, "    ORIGIN Type = ");
 				getroute.origin_type = *ptr;
+				v6getroute.origin_type = *ptr;
 				switch (getroute.origin_type) {
 				case ORIGIN_IGP:
 					strcat(logstr, "IGP\n");
@@ -372,14 +457,15 @@ void pr_update(int pn)
 					"    AS_PATH Not Presented\n", OFF);
 					break;
 				}
-				ATRFL_SET(SET_PATHTYPE,
-						getroute.attribute_flag);
+				ATRFL_SET(SET_PATHTYPE, getroute.attribute_flag);
+				ATRFL_SET(SET_PATHTYPE, v6getroute.attribute_flag);
 				ptmp = 0;
 				/* getroute.path_set[0] = '\0'; */
 				pathbuf[0] = '\0';
 				while(ptmp < attr_octs) {
 				  strcpy(logstr, "    PATH Segment Type = ");
 				  getroute.path_type = *(ptr+ptmp);
+				  v6getroute.path_type = *(ptr+ptmp);
 				  ptmp++;
 				  switch(getroute.path_type){
 			  	  case AS_SET:
@@ -437,10 +523,13 @@ void pr_update(int pn)
 				/* Add 16 Sep 2001 Dynamic Path set */
 				if (strlen(pathbuf) == 0) {
 					getroute.pathset_ptr = NULL;
+					v6getroute.pathset_ptr = NULL;
 					sprintf(logstr, "    AS_Path = N/A\n");
 				} else {
 					getroute.pathset_ptr = &pathbuf[0];
+					v6getroute.pathset_ptr = &pathbuf[0];
 					strcpy(getroute.pathset_ptr, pathbuf);
+					strcpy(v6getroute.pathset_ptr, pathbuf);
 					/*
 					sprintf(logstr, "    AS_Path = %s\n",
 								getroute.path_set);
@@ -452,10 +541,12 @@ void pr_update(int pn)
 				ptr += attr_octs;
 				break;
 			case ATTR_NEXT_HOP:
-				ATRFL_SET(SET_NEXTHOP,
-					getroute.attribute_flag);
+				ATRFL_SET(SET_NEXTHOP, getroute.attribute_flag);
+				ATRFL_SET(SET_NEXTHOP, v6getroute.attribute_flag);
 				(void)memcpy((char *)&getroute.next_hop,
 					ptr, 4);
+				(void)memcpy((char *)&v6getroute.next_hop,
+					ptr, 4);
 				inaddr.s_addr = getroute.next_hop;
 				sprintf(logstr, "    NEXT_HOP Address = %s\n",
 							inet_ntoa(inaddr));
@@ -463,8 +554,8 @@ void pr_update(int pn)
 				ptr += attr_octs;
 				break;
 			case ATTR_MULTI_EXIT_DISC:
-				ATRFL_SET(SET_METRIC, 
-					getroute.attribute_flag);
+				ATRFL_SET(SET_METRIC, getroute.attribute_flag);
+				ATRFL_SET(SET_METRIC, v6getroute.attribute_flag);
 				getroute.metric = buf2ulong(ptr);
 				sprintf(logstr, "    MULTI_EXIT_DISC = %lu\n",
 							getroute.metric);
@@ -472,28 +563,33 @@ void pr_update(int pn)
 				ptr += attr_octs;
 				break;
 			case ATTR_LOCAL_PREF:
-				ATRFL_SET(SET_LOCALPREF,
-						getroute.attribute_flag);
+				ATRFL_SET(SET_LOCALPREF, getroute.attribute_flag);
+				ATRFL_SET(SET_LOCALPREF, v6getroute.attribute_flag);
 				getroute.local_perf = buf2ulong(ptr);
+				v6getroute.local_perf = buf2ulong(ptr);
 				sprintf(logstr, "    Local Preferance = %lu\n",
 							getroute.local_perf);
 				pr_log(VLOG_UPAT, pn, logstr, OFF);
 				ptr += attr_octs;
 				break;
 			case ATTR_ATOMIC_AGGREGATE:
-				ATRFL_SET(SET_ATOMICAGGREGATE,
-						getroute.attribute_flag);
+				ATRFL_SET(SET_ATOMICAGGREGATE, getroute.attribute_flag);
+				ATRFL_SET(SET_ATOMICAGGREGATE, v6getroute.attribute_flag);
 				pr_log(VLOG_UPAT, pn, 
 					"    Atomic Aggregate ON\n",OFF);
 				/* length 0 Data */
 				break;
 			case ATTR_AGGREGATOR:
-				ATRFL_SET(SET_AGGREGATOR, 
-						getroute.attribute_flag);
+				ATRFL_SET(SET_AGGREGATOR, getroute.attribute_flag);
+				ATRFL_SET(SET_AGGREGATOR, v6getroute.attribute_flag);
 				getroute.aggregate_as = buf2ushort(ptr);
+				v6getroute.aggregate_as = buf2ushort(ptr);
 				(void)memcpy(
 					(char *)&getroute.aggregate_address,
 					ptr+2, 4);
+				(void)memcpy(
+					(char *)&v6getroute.aggregate_address,
+					ptr+2, 4);
 				sprintf(logstr, "    Aggregate AS = %d : ",
 						getroute.aggregate_as);
 				pr_log(VLOG_UPAT, pn, logstr, OFF);
@@ -504,15 +600,18 @@ void pr_update(int pn)
 				ptr += attr_octs;
 				break;
 			case ATTR_COMMUNITIES:
-				ATRFL_SET(SET_COMMUNITIES, 
-						getroute.attribute_flag);
+				ATRFL_SET(SET_COMMUNITIES, getroute.attribute_flag);
+				ATRFL_SET(SET_COMMUNITIES, v6getroute.attribute_flag);
 				cnt = 0;
 				getroute.comlist_cnt = 0;
+				v6getroute.comlist_cnt = 0;
 				pr_log(VLOG_UPAT, pn,
 					"    Community List = ", OFF);
 				while((cnt*4) < attr_octs) {
 					getroute.communities[cnt] = 
 					    buf2ulong(ptr + cnt * 4);
+					v6getroute.communities[cnt] = 
+					    buf2ulong(ptr + cnt * 4);
 					sprintf(logstr, "0x%08X(%d:%d) ",
 						getroute.communities[cnt],
 				(u_short)(getroute.communities[cnt] >> 16),
@@ -525,15 +624,18 @@ void pr_update(int pn)
 						break;
 					}
 					getroute.comlist_cnt++;
+					v6getroute.comlist_cnt++;
 				}
 				pr_log(VLOG_UPAT, pn, "\n", OFF);
 				ptr += attr_octs;
 				break;
 			case ATTR_ORIGINATOR_ID:
-				ATRFL_SET(SET_ORIGINATOR,
-						getroute.attribute_flag);
+				ATRFL_SET(SET_ORIGINATOR, getroute.attribute_flag);
+				ATRFL_SET(SET_ORIGINATOR, v6getroute.attribute_flag);
 				(void)memcpy((char *)&getroute.originator_id,
 					ptr, 4);
+				(void)memcpy((char *)&v6getroute.originator_id,
+					ptr, 4);
 				inaddr.s_addr = getroute.originator_id;
 				sprintf(logstr, "    Originator ID = %s\n",
 							inet_ntoa(inaddr));
@@ -541,14 +643,17 @@ void pr_update(int pn)
 				ptr += attr_octs;
 				break;
 			case ATTR_CLUSTER_LIST:
-				ATRFL_SET(SET_CLUSTER_LIST,
-						getroute.attribute_flag);
+				ATRFL_SET(SET_CLUSTER_LIST, getroute.attribute_flag);
+				ATRFL_SET(SET_CLUSTER_LIST, v6getroute.attribute_flag);
 				cnt = 0;
 				pr_log(VLOG_UPAT, pn, 
 					"    Cluster List = ", OFF);
 				while((cnt*4) < attr_octs) {
 					(void)memcpy(
-					(char *)&getroute.cluster_list[cnt],
+						(char *)&getroute.cluster_list[cnt],
+						ptr + cnt * 4, 4);
+					(void)memcpy(
+						(char *)&v6getroute.cluster_list[cnt],
 						ptr + cnt * 4, 4);
 					inaddr.s_addr = 
 						getroute.cluster_list[cnt];
@@ -574,14 +679,14 @@ void pr_update(int pn)
 				break;
 
 			case ATTR_MP_REACH_NLRI: /* RFC2283 */
-				ipv6updcnt = pr_mp_reach_nlri(ptr, attr_octs, pn);
+				ipv6updcnt = pr_mp_reach_nlri(ptr, attr_octs, &v6update, pn);
 				sprintf(logstr, "    Received IPv6 Update Routes = %d\n", ipv6updcnt);
 				pr_log(VLOG_UPAT, pn, logstr, OFF);
 				ptr += attr_octs;
 				break;
 
 			case ATTR_MP_UNREACH_NLRI: /* RFC2283 */
-				ipv6wdrcnt = pr_mp_unreach_nlri(ptr, attr_octs, pn);
+				ipv6wdrcnt = pr_mp_unreach_nlri(ptr, attr_octs, &v6withdraw, pn);
 				sprintf(logstr, "    Received IPv6 Withdrawn Routes = %d\n", ipv6wdrcnt);
 				pr_log(VLOG_UPAT, pn, logstr, OFF);
 				ptr += attr_octs;
@@ -673,5 +778,78 @@ void pr_update(int pn)
 #endif
 	}
 
+	if (ipv6updcnt != 0) {
+		gettimeofday(&v6getroute.lastupdate, (struct timezone *)0);
+		gettimeofday(&v6getroute.firstupdate, (struct timezone *)0);
+		store_int32(&v6getroute.source, &peer[pn].neighbor);
+		v6getroute.src_afi = AF_IPV4;
+		pr_log(VLOG_UPDE, pn, "IPv6 Routes\n", OFF);
+		pr_log(VLOG_UPDE, pn,
+"   Prefix           Next Hop         Metric Loc_Pef Time    AS_Path\n", OFF);
+
+		cnt = 0;
+		if (v6update.preflist != NULL) {
+			listbuf = v6update.preflist;
+
+			/* Next Hop Length Detection */
+			if (*(u_long *)(&v6getroute.next_hop[0]) == 0) {
+				v6getroute.nexthop_afi = AF_IPV6;
+				store_v6addr(v6getroute.next_hop, v6update.nexthop);
+				ATRFL_SET(SET_NEXTHOP, v6getroute.attribute_flag);
+			} else {
+				v6getroute.nexthop_afi = AF_IPV4;
+			}
+
+		} else {
+			ipv6updcnt = 0;
+		}
+		while(cnt < ipv6updcnt) {
+			bzero(&v6prroute, sizeof(ROUTE_INFO_V6));
+			memmove(&v6prroute, &v6getroute, sizeof(ROUTE_INFO_V6)); 
+
+			v6prroute.length = listbuf->length;
+
+			if (prroute.length > (IPV6_ADDR_LEN*8)) {
+				pt2++;
+				pr_log(VLOG_UPDE, pn, "IPv6 Prefix too long.\n", OFF);
+				continue;
+			}
+	
+			bzero(v6prroute.prefix, IPV6_ADDR_LEN);
+			store_v6addr(v6prroute.prefix, listbuf->v6prefix);
+	
+			if (v6prroute.pathset_ptr != NULL) {
+		    	pathbuflen = strlen(v6getroute.pathset_ptr);
+		    	if (pathbuflen != 0) {
+				v6prroute.pathset_ptr = (char *)malloc(pathbuflen+1);
+				if (v6prroute.pathset_ptr == NULL) {
+			    	pr_log(VLOG_GLOB, LOG_ALL, 
+			        	"pr_update: pathset_ptr: Malloc error\n", ON);
+			    	notify_systemnotify("REBOOT",
+			    				"Malloc Error : pr_update");
+			    	finish(PROC_REBOOT);
+			    	return;
+				}
+		    	}
+		    	strcpy(v6prroute.pathset_ptr, v6getroute.pathset_ptr);
+			}
+
+			pr_v6route(&v6prroute, pn);
+			write_v6prefixlog(0, &v6prroute, pn);
+
+			listbuf = listbuf->next;
+			cnt++;
+	
+
+		}
+		bgpinfo[pn].prefixcnt += ipv6updcnt;
+	}
+	if (ipv6wdrcnt != 0) {
+		write_v6prefixlog(1, &v6withdraw, pn);
+		bgpinfo[pn].withdrawcnt += ipv6wdrcnt;
+	}
+	free_v6routebuf(&v6update);
+	free_v6routebuf(&v6withdraw);
+
 	pr_log(VLOG_UPDE, pn, "===== UPDATE Information Done =====\n", ON);
 }
