@@ -4,7 +4,7 @@
 /*                                                              */
 /* Utility functions                                            */
 /****************************************************************/
-/* utils.c,v 1.8 2002/03/28 09:54:37 kuniaki Exp */
+/* utils.c,v 1.9 2002/04/09 09:43:06 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -292,3 +292,7 @@ char *strtobinv6(char *ptr, char *str)
         return(ptr);
 }
 
+void store_v6addr(char *dest, char *src)
+{
+	memcpy(dest, src, IPV6_ADDR_LEN);
+}
