@@ -4,7 +4,7 @@
 /*                                                              */
 /* Utility Sub-routines                                         */
 /****************************************************************/
-/* utils.h,v 1.5 2002/03/28 09:54:37 kuniaki Exp */
+/* utils.h,v 1.6 2002/04/09 09:43:06 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -43,16 +43,14 @@
 #include <string.h>
 #include <pwd.h>
 #include <unistd.h>
-
+#include "bvtypes.h"
+#include "route.h"
+#include "route_ipv6.h"
 
 
 #define OFF                     0
 #define ON                      1
 
-typedef char                    boolean;
-#define true                    1
-#define false                   0
-
 #define CHKFMT_ADR	1
 #define CHKFMT_PRF	2
 #define CHKFMT_INT	3
@@ -71,5 +69,6 @@ extern int	checkpw(/* char *phrase, char *pass */);
 extern void 	wait_sigchld();
 extern int 	bintostrv6(/* char *ptr, char *str */);
 extern char 	*strtobinv6(/* char *ptr, char *str */);
+extern void	store_v6addr(/* char *dest, char *src */);
 
 #endif /* _UTILS_H */
